#include <qlayout.h>
#include <qframe.h>

#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>

#include "configdialog.h"
#include "gestureconfigpage.h"

ConfigDialog::ConfigDialog(QWidget *parent, const char *name)
  : KDialogBase(KDialogBase::Plain, i18n("Configuration"),
		KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel,
		KDialogBase::Ok, parent, name, true, true), _gcPage(NULL)
  {
  // The apply button will be enabled when something is modified
  enableButton(KDialogBase::Apply, false);

  QFrame *page = plainPage();
  QVBoxLayout *layout = new QVBoxLayout(page);
  
  _gcPage = new GestureConfigPage(page, "gestureConfigPage");
  _gcPage->setMinimumSize(_gcPage->sizeHint());
  connect(_gcPage, SIGNAL(modified()),
	  this, SLOT(slotModified()));
  layout->addWidget(_gcPage);
  
  page->setMinimumSize(page->sizeHint());
  }

ConfigDialog::~ConfigDialog()
  {
  }

void ConfigDialog::readOptions(KConfig &config)
  {
  _gcPage->readOptions(config);
  }

void ConfigDialog::writeOptions(KConfig &config)
  {
  _gcPage->writeOptions(config);
  }

void ConfigDialog::slotApply()
  {
  KConfig *config = kapp->config();
  writeOptions(*config);

  enableButton(KDialogBase::Apply, false);
  }

void ConfigDialog::slotOk()
  {
  slotApply();
  
  KDialogBase::slotOk();
  }

void ConfigDialog::slotModified()
  {
  enableButton(KDialogBase::Apply, true);
  }
