#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <klocale.h>
#include <kmessagebox.h>

#include "gesturerecordpage.h"
#include "gesturerecorder.h"
#include "gesturedrawer.h"
#include "gesture.h"

GestureRecordPage::GestureRecordPage(QList<Gesture> &gList,
		    QWidget *parent, const char *name)
  : QVBox(parent, name),
    _gestureList(gList), _recorder(NULL), _resetButton(NULL),
    _tryOne(NULL), _tryTwo(NULL), _tryThree(NULL), _gest(NULL),
    _tryCount(1)
  {
  QString message;

  message += "Draw the gesture you would like to record below. Press\n";
  message += "and hold the left mouse button while drawing, and release\n";
  message += "when your done. \n\n";

  message += "You will be required to draw the gesture 3 times. After\n";
  message += "each drawing, if they match, the indicators below will\n";
  message += "change to represent which step you are on.\n\n";

  message += "If at any point they do not match, you will be required to \n";
  message += "restart. If you want to force a restart, use the restart \n";
  message += "button below.\n\nDraw here:";

  (void) new QLabel(i18n(message), this, "label");

  _recorder = new GestureRecorder(this, "recorder");
  _recorder->setMinimumHeight(150);
  setStretchFactor(_recorder, 1);
  connect(_recorder, SIGNAL(recorded(const QString &)),
	  this, SLOT(slotRecorded(const QString &)));

  QHBox *hBox = new QHBox(this, "hbox");

  _tryOne = new GestureDrawer(hBox, "tryOne");
  _tryOne->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  _tryOne->setMinimumSize(30, 20);
  
  _tryTwo = new GestureDrawer(hBox, "tryTwo");
  _tryTwo->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  _tryTwo->setMinimumSize(30, 20);
 
  _tryThree = new GestureDrawer(hBox, "tryThree");
  _tryThree->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  _tryThree->setMinimumSize(30, 20);
 
//   _tryOne = new QLabel(i18n("1"), hBox, "tryOne");
//   _tryOne->setBackgroundColor(red);
//   _tryOne->setMinimumSize(30, 20);
//   _tryOne->setAlignment(AlignCenter);
//   _tryOne->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  
//   _tryTwo = new QLabel(i18n("2"), hBox, "tryTwo");
//   _tryTwo->setBackgroundColor(red);
//   _tryTwo->setMinimumSize(30, 20);
//   _tryTwo->setAlignment(AlignCenter);
//   _tryTwo->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  
//   _tryThree = new QLabel(i18n("3"), hBox, "tryThree");
//   _tryThree->setBackgroundColor(red);
//   _tryThree->setMinimumSize(30, 20);
//   _tryThree->setAlignment(AlignCenter);
//   _tryThree->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  
  QWidget *spacer = new QWidget(hBox, "spacer");
  hBox->setStretchFactor(spacer, 1);

  _resetButton = new QPushButton(i18n("&Reset"), hBox, "resetButton");
  connect(_resetButton, SIGNAL(clicked()),
	  this, SLOT(slotResetClicked()));
  }

GestureRecordPage::~GestureRecordPage()
  {
  }

void GestureRecordPage::slotRecorded(const QString &data)
  {
  switch (_tryCount)
    {
    case 1:
	_gest = new Gesture(data);
	_tryOne->setData(data);
	_tryCount++;
	break;

    case 2:
     {
     Gesture g(data);
     if (*_gest == data)
       {
       _tryTwo->setData(data);
       _tryCount++;
       }
     else
       {
       KMessageBox::sorry(this, "Sorry, your gestures didn't match.");
       slotResetClicked();
       }
     break;
     }
    case 3:
    {
     Gesture g(data);
     if (*_gest == data)
       {
       _tryThree->setData(data);
       _tryCount++;
       emit readyForNext(true);
       }
     else
       {
       KMessageBox::sorry(this, i18n("Sorry, your gestures didn't match."));
       slotResetClicked();
       }
     break;
     }
    default:
	KMessageBox::information(this, i18n("You have already completed the three required drawings. Either press Next to move on or Reset to try again."));
    }
  }

void GestureRecordPage::slotResetClicked()
  {
  delete _gest;
  _gest = NULL;
  
  _tryOne->setData(QString::null);
  _tryTwo->setData(QString::null);
  _tryThree->setData(QString::null);

  _tryCount = 1;

  emit readyForNext(false);
  }
