#include <qcolor.h>
#include <qpainter.h>

#include "gesturedrawer.h"

GestureDrawer::GestureDrawer(QWidget *parent, const char *name)
  : QFrame(parent, name), _data(QString::null)
  {
  setBackgroundColor(white);
  }

GestureDrawer::~GestureDrawer()
  {
  }

void GestureDrawer::setData(const QString &data)
  {
  _data = data;

  repaint();
  }

void GestureDrawer::paintEvent(QPaintEvent *ev)
  {
  // Iterate through the data points and draw a line to each of them
  Q_UINT32 startCell = 0;
  Q_UINT32 endCell = 0;
  QPoint startPoint;
  QPoint endPoint;

  QPainter p(this);
  
  if (_data.length() > 0)
    startCell = QString(_data[0]).toUInt();
  
  for (Q_UINT32 index = 1; index < _data.length(); ++index)
    {
    endCell = QString(_data[index]).toUInt();

    startPoint = lookupCellCoords(startCell);
    endPoint = lookupCellCoords(endCell);

    p.drawLine(startPoint, endPoint);

    startCell = endCell;
    }

  p.end();

  QFrame::paintEvent(ev);
  }

QPoint GestureDrawer::lookupCellCoords(Q_UINT32 cell)
  {
  // First divide the widget into thirds, horizontally and vertically
  Q_UINT32 w = width();
  Q_UINT32 h = height();

  Q_UINT32 wThird = w / 3;
  Q_UINT32 hThird = h / 3;

  switch(cell)
    {
    case 1:
	return QPoint(wThird/2, hThird/2);

    case 2:
	return QPoint(wThird+wThird/2, hThird/2);
	
    case 3:
	return QPoint(2*wThird+wThird/2, hThird/2);
	
    case 4:
	return QPoint(wThird/2, hThird+hThird/2);
	
    case 5:
	return QPoint(wThird+wThird/2, hThird+hThird/2);
	
    case 6:
	return QPoint(2*wThird+wThird/2, hThird+hThird/2);
	
    case 7:
	return QPoint(wThird/2, 2*hThird+hThird/2);
	
    case 8:
	return QPoint(wThird+wThird/2, 2*hThird+hThird/2);
	
    case 9:
	return QPoint(2*wThird+wThird/2, 2*hThird+hThird/2);
    }

  return QPoint(0, 0);
  }
