#include <kdebug.h>
#include <kprocess.h>

#include "gesturerecognizer.h"
#include "gesture.h"

GestureRecognizer::GestureRecognizer(QObject *parent, const char *name)
  : QObject(parent, name), _gestureList()
  {
  _gestureList.setAutoDelete(true);
  }

GestureRecognizer::~GestureRecognizer()
  {
  }

void GestureRecognizer::lookupGesture(const Gesture &g)
  {
  Gesture *gest = NULL;
  bool found = false;
  QString action;
  QListIterator<Gesture> iter(_gestureList);

  // Iterator through the list of gestures and try to find it
  for (iter.toFirst(); iter.current() && !found; ++iter)
    {
    gest = iter.current();

    if (*gest == g)
      {
      found = true;
      }
    }

  if (found)
    {
    // Try to run the action
    action = gest->getAction();

    if ((action != QString::null) && (action.length() > 0))
      {
      kdDebug() << "GestureRecognizer::lookupGesture: " << action << "\n";
      
      // Run it here
      KProcess process;
      Q_UINT32 start = 0;
      int length = 0;
      
      while ((length = action.find(" ", start)) != -1)
	{
	length = length-start;
	//kdDebug() << action.mid(start, length) << "\n";
	process << action.mid(start, length);

	start += length + 1;
	}

      process << action.mid(start, action.length()-start);
      process.start(KProcess::DontCare, KProcess::NoCommunication);
      }
    }
  }

void GestureRecognizer::readOptions(KConfig &config)
  {
  _gestureList.clear();

  _gestureList = Gesture::readGestures(config);
  }

void GestureRecognizer::writeOptions(KConfig &config)
  {
  Gesture::writeGestures(_gestureList, config);
  }
